
/*     *******************************************************************************
	XVisual GUI Development System for Linux/X-Window
	Copyright (C) 1996 Steve Carrie.

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
        ******************************************************************************* */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw3d/Command.h>
#include <X11/xv/xvCore.h>
#include <X11/xv/xvFont.h>
#include <X11/xv/xvBasicBitmap.h>
#include <X11/xv/xvButton.h>

// -----------------------------------------------------------------

xvButton::xvButton()
{
	alloc=FALSE;
}

// -----------------------------------------------------------------

xvButton::xvButton(char *name, xvCore & parent, XtCallbackProc callback, String client)
{
	init(name, parent, callback, client);
}

// =================================================================

xvButton::~xvButton()
{
	if (alloc) XtDestroyWidget(w);
}

// -----------------------------------------------------------------

void  xvButton::init(char *name, xvCore & parent, XtCallbackProc callback, String client)
{
	init(name, parent.ID(), callback, client);
}

// -----------------------------------------------------------------
void  xvButton::init(char *name, Widget parent, XtCallbackProc callback, String client)
{
	w=XtVaCreateManagedWidget(name, commandWidgetClass, parent, 
	    NULL);
	alloc=TRUE;          
	XtAddCallback(w, XtNcallback, callback, (XtPointer) client);
	bfont.init(w, NULL);
	bmap.init(NULL, w);
}

// -----------------------------------------------------------------

void xvButton::ChangeFont(String fontname)
{
	if (fontname != NULL) {
		bfont.FLoadFont(fontname);
		XtVaSetValues(w, XtNfont, (XtArgVal) bfont.FontRef(), NULL);
	}
}

// -----------------------------------------------------------------

int xvButton::ChangeBitmap(String name, int pixtype, int position)
{
	int err;

	if ((err=bmap.ReadFile(name, pixtype)) == 0)
		if (position==XVCENTREBITMAP)
			XtVaSetValues(w, XtNbitmap, (XtArgVal) bmap.GetPixmap(), NULL);
		else
			XtVaSetValues(w, XtNleftBitmap, (XtArgVal) bmap.GetPixmap(), NULL);
	return(err);
}

// -----------------------------------------------------------------
	