
/*     *******************************************************************************
	XVisual GUI Development System for Linux/X-Window
	Copyright (C) 1996 Steve Carrie.

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
        ******************************************************************************* */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/xpm.h>
#include <X11/Xaw3d/ThreeD.h>
#include <X11/xv/xvCore.h>
#include <string.h>

// ---------------------------------

xvCore::~xvCore()
{
}

// ---------------------------------

inline void xvCore::Hide() 
{
	XtUnrealizeWidget(w);
}

// ---------------------------------

inline void xvCore::Show()
{
	XtRealizeWidget(w);
	XtMapWidget(w);
}

// ---------------------------------

inline void xvCore::Unmap() 
{
	XtUnmapWidget(w);
}

// ---------------------------------

inline void xvCore::Map()
{
	XtMapWidget(w);
}

// ---------------------------------

inline void xvCore::SetAccelerators(XtAccelerators atable)
{
	XtVaSetValues(w, XtNaccelerators, (XtArgVal) atable, NULL);
}

//----------------------------------

inline void xvCore::SetAncestorSensitive(Boolean sense)
{
	XtVaSetValues(w, XtNancestorSensitive, (XtArgVal) sense, NULL);
}

// ---------------------------------

inline void xvCore::SetBackground(Pixel b)
{
	if (b!=-1)
		XtVaSetValues(w, XtNbackground, (XtArgVal) b, NULL);
}

// ----------------------------------

inline void xvCore::SetBackground(String colour)
{	
	XtVaSetValues(w, XtVaTypedArg, XtNbackground, XtRString, 
				colour, strlen(colour), NULL);
}

// ----------------------------------

inline void xvCore::SetBackgroundPixmap(Pixmap p)
{
	if (p != -1) 
		XtVaSetValues(w, XtNbackgroundPixmap, (XtArgVal) p, NULL);
}

// ---------------------------------

inline void xvCore::SetBorderColor(Pixel b)
{
	if (b != -1)
		XtVaSetValues(w, XtNborderColor, (XtArgVal) b, NULL);
}

// ----------------------------------

inline void xvCore::SetBorderColor(String colour)
{	
	XtVaSetValues(w, XtVaTypedArg, XtNborderColor, XtRString, 
				colour, strlen(colour), NULL);
}

// ----------------------------------

inline void xvCore::SetBorderPixmap(Pixmap p)
{
	if (p != -1) 
		XtVaSetValues(w, XtNborderPixmap, (XtArgVal) p, NULL);
}

// ---------------------------------

inline void xvCore::SetBorderWidth(Dimension b)
{
	XtVaSetValues(w, XtNborderWidth, (XtArgVal) b, NULL);
}

// ---------------------------------

inline Dimension xvCore::GetBorderWidth()
{
	Dimension bw;
	XtVaGetValues(w, XtNborderWidth, &bw, NULL);
	return(bw);
}

// ---------------------------------

inline void xvCore::SetColormap(Colormap c)
{
	XtVaSetValues(w, XtNcolormap, (XtArgVal) c, NULL);
}

// ---------------------------------

inline void xvCore::SetDepth(int d)
{
	XtVaSetValues(w, XtNdepth, (XtArgVal) d, NULL);
}

// ----------------------------------

inline int xvCore::GetDepth()
{
	int d;
	XtVaGetValues(w, XtNdepth, &d, NULL);
	return(d);
}

// ----------------------------------

inline void xvCore::SetDestroyCallback(XtCallbackProc d)
{
	XtAddCallback(w, XtNdestroyCallback, d, 0);
}

// ----------------------------------

inline void xvCore::SetHeight(Dimension h)
{
	XtVaSetValues(w, XtNheight, (XtArgVal) h, NULL);
}

// ----------------------------------

inline Dimension xvCore::GetHeight()
{
	Dimension h;
	XtVaGetValues(w, XtNheight, &h, NULL);
	return(h);
}

// ----------------------------------

inline void xvCore::SetMappedWhenManaged(Boolean m)
{
	XtVaSetValues(w, XtNmappedWhenManaged, (XtArgVal) m, NULL);
}

// ----------------------------------

inline void xvCore::SetSensitive(Boolean s)
{
	XtVaSetValues(w, XtNsensitive, (XtArgVal) s, NULL);
}

// ----------------------------------

inline void xvCore::SetTranslations(XtTranslations trans)
{
	XtVaSetValues(w, XtNtranslations, (XtArgVal) trans, NULL);
}

// ----------------------------------

inline void xvCore::OverrideTranslations(XtTranslations trans)
{
	XtOverrideTranslations(w, trans);
}

// ----------------------------------

inline void xvCore::AugmentTranslations(XtTranslations trans)
{
	XtAugmentTranslations(w, trans);
}

// ----------------------------------

inline void xvCore::SetWidth(Dimension xw)
{
	XtVaSetValues(w, XtNwidth, (XtArgVal) xw, NULL);
}

// ----------------------------------

inline Dimension xvCore::GetWidth()
{
	Dimension xw;
	XtVaGetValues(w, XtNwidth, &xw, NULL);
	return(xw);
}

// ----------------------------------

inline void xvCore::SetX(Position x)
{
	XtVaSetValues(w, XtNx, (XtArgVal) x, NULL);
}

// ----------------------------------

inline Position xvCore::GetX()
{
	Position x;
	XtVaGetValues(w, XtNx, &x, NULL);
	return(x);
}

// ----------------------------------

inline void xvCore::SetY(Position y)
{
	XtVaSetValues(w, XtNy, (XtArgVal) y, NULL);
}

// ---------------------------------

inline Position xvCore::GetY()
{
	Position y;
	XtVaGetValues(w, XtNy, &y, NULL);
	return(y);
}

// ----------------------------------

inline void xvCore::SetXY(Position x, Position y)
{
	XtVaSetValues(w, XtNx, (XtArgVal) x, XtNy, (XtArgVal) y, NULL);
}

// ---------------------------------

inline void xvCore::GetXY(Position *x, Position *y)
{
	XtVaGetValues(w, XtNx, x, XtNy, y, NULL);
}

// ---------------------------------

inline void xvCore::SetResource(String rname, XtArgVal val)
{
	XtVaSetValues(w, rname, (XtArgVal) val, NULL);
}

// ---------------------------------

inline void xvCore::GetResource(String rname, XtArgVal *val)
{
	XtVaGetValues(w, rname, val, NULL);
}

// ---------------------------------

inline void xvCore::SetShadowWidth(Dimension sw)
{
	XtVaSetValues(w, XtNshadowWidth, (XtArgVal) sw, NULL);
}

// ---------------------------------

inline void xvCore::SetTopShadowPixel(Pixel tsp)
{
	if (tsp != -1)
		XtVaSetValues(w, XtNtopShadowPixel, (XtArgVal) tsp, NULL);
}

// ---------------------------------

inline void xvCore::SetTopShadowPixel(String colour)
{	
	XtVaSetValues(w, XtVaTypedArg, XtNtopShadowPixel, XtRString, 
				colour, strlen(colour), NULL);
}

// ---------------------------------

inline void xvCore::SetBottomShadowPixel(Pixel bsp)
{
	if (bsp != -1)
		XtVaSetValues(w, XtNbottomShadowPixel, (XtArgVal) bsp, NULL);
}

// ---------------------------------

inline void xvCore::SetBottomShadowPixel(String colour)
{	
	XtVaSetValues(w, XtVaTypedArg, XtNbottomShadowPixel, XtRString, 
				colour, strlen(colour), NULL);
}

// ---------------------------------

inline void xvCore::SetTopShadowContrast(int tsc)
{
	XtVaSetValues(w, XtNtopShadowContrast, (XtArgVal) tsc, NULL);
}

// ---------------------------------

inline void xvCore::SetBottomShadowContrast(int bsc)
{
	XtVaSetValues(w, XtNbottomShadowContrast, (XtArgVal) bsc, NULL);
}

// ---------------------------------

inline void xvCore::SetUserData(XtPointer ud)
{
	XtVaSetValues(w, XtNuserData, (XtArgVal) ud, NULL);
}

// ---------------------------------

inline void xvCore::SetBeNiceToColormap(Boolean bntc)
{
	XtVaSetValues(w, XtNbeNiceToColormap, (XtArgVal) bntc, NULL);
}

// ---------------------------------

inline Widget xvCore::ID()
{
	return(w);
}

// ---------------------------------

inline void xvCore::SetLabel(String lab)
{
	XtVaSetValues(w, XtNlabel, (XtArgVal) lab, NULL);
}

// ---------------------------------	

inline void xvCore::SetFont(XFontStruct *xfs)
{
	XtVaSetValues(w, XtNfont, (XtArgVal) xfs, NULL);
}

// --------------------------------

inline void xvCore::SetForeground(Pixel fg)
{
	if (fg != -1)
		XtVaSetValues(w, XtNforeground, (XtArgVal) fg, NULL);
}

// --------------------------------

inline void xvCore::SetForeground(String colour)
{	
	XtVaSetValues(w, XtVaTypedArg, XtNforeground, XtRString, 
				colour, strlen(colour), NULL);
}

// --------------------------------

inline void xvCore::SetFontSet(XFontSet xfnt)
{
	XtVaSetValues(w, XtNfontSet, (XtArgVal) xfnt, NULL);
}

// --------------------------------

inline void xvCore::SetJustify(XtJustify xj)
{
	XtVaSetValues(w, XtNjustify, (XtArgVal) xj, NULL);
}

// --------------------------------

inline void xvCore::SetCallback(XtCallbackProc callback)
{
	XtVaSetValues(w, XtNcallback, (XtArgVal) callback, NULL);
}

// --------------------------------

inline void xvCore::SetHSpace(Dimension hs)
{
	XtVaSetValues(w, XtNhSpace, (XtArgVal) hs, NULL);
}

// --------------------------------

inline void xvCore::SetVSpace(Dimension vs)
{
	XtVaSetValues(w, XtNvSpace, (XtArgVal) vs, NULL);
}

// --------------------------------

inline void xvCore::SetBottomEdge(XawEdgeType bedge)
{
	XtVaSetValues(w, XtNbottom, (XtArgVal) bedge, NULL);
}

// --------------------------------

inline void xvCore::SetLeftEdge(XawEdgeType ledge)
{
	XtVaSetValues(w, XtNleft, (XtArgVal) ledge, NULL);
}

// --------------------------------

inline void xvCore::SetRightEdge(XawEdgeType redge)
{
	XtVaSetValues(w, XtNright, (XtArgVal) redge, NULL);
}

// --------------------------------

inline void xvCore::SetTopEdge(XawEdgeType tedge)
{
	XtVaSetValues(w, XtNtop, (XtArgVal) tedge, NULL);
}

// --------------------------------

inline void xvCore::SetRelHorizontal(xvCore & wg)
{
	XtVaSetValues(w, XtNfromHoriz, (XtArgVal) wg.ID(), NULL);
}

// -------------------------------

inline void xvCore::SetRelVertical(xvCore &  wg)
{
	XtVaSetValues(w, XtNfromVert, (XtArgVal) wg.ID(), NULL);
}

// -------------------------------

inline void xvCore::SetRelative(xvCore & hg, xvCore & vg)
{
	XtVaSetValues(w, XtNfromHoriz, (XtArgVal) hg.ID(), XtNfromVert, (XtArgVal) vg.ID(), NULL);
}

// -------------------------------

inline void xvCore::SetResizable(Boolean srz)
{
	XtVaSetValues(w, XtNresizable, (XtArgVal) srz, NULL);
}

// -------------------------------

inline void xvCore::SetVertDistance(int vd)
{
	XtVaSetValues(w, XtNvertDistance, (XtArgVal) vd, NULL);
}

// -------------------------------

inline int xvCore::GetVertDistance()
{
	int vd;

	XtVaGetValues(w, XtNvertDistance, &vd, NULL);
	return(vd);
}

// -------------------------------

inline void xvCore::SetHorizDistance(int hd)
{
	XtVaSetValues(w, XtNhorizDistance, (XtArgVal) hd, NULL);
}

// -------------------------------

inline int xvCore::GetHorizDistance()
{
	int vd;

	XtVaGetValues(w, XtNhorizDistance, &vd, NULL);
	return(vd);
}

// -------------------------------

inline void xvCore::SetDistance(int hd, int vd)
{
	XtVaSetValues(w, XtNhorizDistance, hd, XtNvertDistance, vd, NULL);
}

// -------------------------------

inline void xvCore::GetDistance(int *hd, int *vd)
{
	XtVaGetValues(w, XtNhorizDistance, hd, XtNvertDistance, vd, NULL);
}

// -------------------------------

inline void xvCore::SetDefaultDistance(int dd)
{
	XtVaSetValues(w, XtNdefaultDistance, (XtArgVal) dd, NULL);
}

// -------------------------------

inline void xvCore::SetIcon(Pixmap ico)
{
	XtVaSetValues(w, "icon", (XtArgVal) ico, NULL);
}

// -------------------------------

inline Pixmap xvCore::SetBitmapInfo(unsigned char *bits, unsigned int sw, unsigned int sh,
			String resource)
{
	icon=XCreateBitmapFromData(XtDisplay(w), 
		RootWindowOfScreen(XtScreen(w)),
		(char *)bits, sw, sh);

	if (resource != NULL)
		XtVaSetValues(w, resource, (XtArgVal) icon, NULL);

	return(icon);
}

// -------------------------------

inline Pixmap xvCore::SetBitmapInfo(char *bits, unsigned int sw, unsigned int sh,
			String resource)
{
	icon=XCreateBitmapFromData(XtDisplay(w), 
		RootWindowOfScreen(XtScreen(w)),
		bits, sw, sh);

	if (resource != NULL)
		XtVaSetValues(w, resource, (XtArgVal) icon, NULL);

	return(icon);
}

// -------------------------------

inline Pixmap xvCore::SetPixmapInfo(char **bits, String resource)
{
	int ret=XpmCreatePixmapFromData(XtDisplay(w), 
		RootWindowOfScreen(XtScreen(w)),
		bits, &icon, &iconmask, NULL);

	if (ret==0)
		if (resource != NULL)
			XtVaSetValues(w, resource, (XtArgVal) icon, NULL);

	return(icon);
}

// -------------------------------

inline void xvCore::SetInternalWidth(Dimension iw)
{
	XtVaSetValues(w, XtNinternalWidth, (XtArgVal) iw, NULL);
}

// -------------------------------

inline Dimension xvCore::GetInternalWidth()
{
	Dimension iw;
	XtVaGetValues(w, XtNinternalWidth, &iw, NULL);
	return(iw);
}

// -------------------------------

inline void xvCore::SetInternalHeight(Dimension ih)
{
	XtVaSetValues(w, XtNinternalHeight, (XtArgVal) ih, NULL);
}

// -------------------------------

inline Dimension xvCore::GetInternalHeight()
{
	Dimension ih;
	XtVaGetValues(w, XtNinternalHeight, &ih, NULL);
	return(ih);
}

// -------------------------------

inline void xvCore::SetGeometry(Position x, Position y, Dimension wid, Dimension h)
{
	XtVaSetValues(w, XtNx, (XtArgVal) x, 
					XtNy, (XtArgVal) y, 
					XtNwidth, (XtArgVal) wid, 
					XtNheight, (XtArgVal) h, 	
					NULL);
}

// -------------------------------

inline void xvCore::GetGeometry(Position *x, Position *y, Dimension *wid, Dimension *h)
{
	XtVaGetValues(w, XtNx, x, XtNy, y, XtNwidth, wid, XtNheight, h, NULL);
}

// --------------------------------

inline void xvCore::SetSize(Dimension wid, Dimension h)
{
	XtVaSetValues(w, XtNwidth, (XtArgVal) wid, 
					XtNheight, (XtArgVal) h, 	
					NULL);
}

// -------------------------------

inline void xvCore::GetSize(Dimension *wid, Dimension *h)
{
	XtVaGetValues(w, XtNwidth, wid, XtNheight, h, NULL);
}

// --------------------------------

inline void xvCore::AddEvent(EventMask eventmask, XtEventHandler handler, XtPointer cdata)
{
	XtAddEventHandler(w, eventmask, FALSE, handler, cdata);
}

// --------------------------------

inline void xvCore::RemoveEvent(EventMask eventmask, XtEventHandler handler, XtPointer cdata)
{
	XtRemoveEventHandler(w, eventmask, FALSE, handler, cdata);
}

// --------------------------------

inline void xvCore::SetTreeGC(GC ngc)
{
	XtVaSetValues(w, XtNtreeGC, (XtArgVal) ngc, NULL);
}

// --------------------------------

inline void xvCore::SetTreeParent(Widget parent)
{
	XtVaSetValues(w, XtNtreeParent, (XtArgVal) parent, NULL);
}

// ---------------------------------

inline void xvCore::SetTreeParent(xvCore & parent)
{
	XtVaSetValues(w, XtNtreeParent, (XtArgVal) parent.ID(), NULL);
}

// ---------------------------------

inline void xvCore::SetBitmap(Pixmap pix)
{
	XtVaSetValues(w, XtNbitmap, (XtArgVal) pix, NULL);
}

// ---------------------------------

inline void xvCore::SetFixedPosition(Boolean fix)
{
	if (fix) 
		XtVaSetValues(w, XtNleft, XawChainLeft, XtNright, XawChainLeft,
					XtNtop, XawChainTop, XtNbottom, XawChainTop, NULL);
	else
		XtVaSetValues(w, XtNleft, XawRubber, XtNright, XawRubber, 
					XtNtop, XawRubber, XtNbottom, XawRubber, NULL);
}

// -----------------------------------

inline void xvCore::SetCursorName(String cname)
{
	XtVaSetValues(w, XtNcursor, None, XtNcursorName, (XtArgVal) NULL, NULL);
	XtVaSetValues(w, XtNcursorName, (XtArgVal) cname, NULL);
}

// ------------------------------------

inline void xvCore::SetCursor(Cursor curs)
{
	XtVaSetValues(w, XtNcursor, (XtArgVal) curs, NULL);
}

// ------------------------------------

inline void xvCore::AddGrab(Boolean exclusive, Boolean sprung)
{
	XtAddGrab(w, exclusive, sprung);
}

// ------------------------------------

inline void xvCore::RemoveGrab()
{
	XtRemoveGrab(w);
}

// ------------------------------------

inline Pixel xvCore::stringToPixel(String colname)
{
	XrmValue from, to;
	Pixel output;
	Boolean testx;

	from.addr=colname;
	from.size=strlen(colname);

	to.addr=(char *) &output;
	to.size=sizeof(Pixel);

	testx=XtConvertAndStore(w, XtRString, &from, XtRPixel, &to);

	if (testx) 
		return(output);	
	else
		return((Pixel) -1);
}

// ----------------------------------------------------
